VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "howto_circle_circle_intersection"
   ClientHeight    =   4140
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   4140
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Const NAN As Single = 1E+20

Private m_Cx(0 To 1) As Single
Private m_Cy(0 To 1) As Single
Private m_Radius(0 To 1) As Single
Private m_NumCircles As Integer

Private m_IntersectionX(0 To 1) As Single
Private m_IntersectionY(0 To 1) As Single
Private m_NumIntersections As Integer

Private m_Drawing As Boolean
Private m_X1 As Single
Private m_Y1 As Single
Private m_X2 As Single
Private m_Y2 As Single

Private Sub Form_MouseDown(Button As Integer, Shift As Integer, x As Single, y As Single)
    If (m_NumCircles = 2) Then
        ' Start over.
        m_NumCircles = 0
        m_NumIntersections = 0
    End If
    
    m_Drawing = True
    m_X1 = x
    m_Y1 = y
    m_X2 = x
    m_Y2 = y

    Me.Cls
    Me.Refresh
End Sub

Private Sub Form_MouseMove(Button As Integer, Shift As Integer, x As Single, y As Single)
    If Not m_Drawing Then Exit Sub

    m_X2 = x
    m_Y2 = y

    Me.Refresh
End Sub

Private Sub Form_MouseUp(Button As Integer, Shift As Integer, x As Single, y As Single)
Dim wid, x0, y0 As Single
Dim radius0, cx0, cy0, radius1, cx1, cy1 As Single

    m_Drawing = False

    ' Save the new circle.
    wid = Min(Abs(m_X1 - m_X2), Abs(m_Y1 - m_Y2))
    x0 = m_X1
    y0 = m_Y1
    If (m_X1 > m_X2) Then x0 = m_X1 - wid
    If (m_Y1 > m_Y2) Then y0 = m_Y1 - wid

    m_Radius(m_NumCircles) = wid / 2
    m_Cx(m_NumCircles) = x0 + wid / 2
    m_Cy(m_NumCircles) = y0 + wid / 2
    m_NumCircles = m_NumCircles + 1

    ' If we have two circles, see where they intersect.
    If (m_NumCircles = 2) Then
        radius0 = m_Radius(0)
        cx0 = m_Cx(0)
        cy0 = m_Cy(0)
        radius1 = m_Radius(1)
        cx1 = m_Cx(1)
        cy1 = m_Cy(1)
        m_NumIntersections = FindCircleCircleIntersections( _
            cx0, cy0, radius0, cx1, cy1, radius1, _
            m_IntersectionX(0), m_IntersectionY(0), _
            m_IntersectionX(1), m_IntersectionY(1))
    End If

    ' Redraw.
    Me.Refresh
End Sub

' Redraw whatever we have.
Private Sub Form_Paint()
Dim i As Integer
Dim wid, x0, y0 As Single

    ' Draw the circles.
    For i = 0 To m_NumCircles - 1
        Circle (m_Cx(i), m_Cy(i)), m_Radius(i)
    Next i

    ' Draw the new circle.
    If (m_Drawing) Then
        wid = Min(Abs(m_X1 - m_X2), Abs(m_Y1 - m_Y2))
        x0 = m_X1
        y0 = m_Y1
        If (m_X1 > m_X2) Then x0 = m_X1 - wid
        If (m_Y1 > m_Y2) Then y0 = m_Y1 - wid
        
        Circle (x0 + wid / 2, y0 + wid / 2), wid / 2
    End If

    ' If we have a solution, display it.
    If (m_NumIntersections >= 1) Then
        DrawPoint m_IntersectionX(0), m_IntersectionY(0), vbRed, vbRed
    End If
    If (m_NumIntersections >= 2) Then
        DrawPoint m_IntersectionX(1), m_IntersectionY(1), vbRed, vbRed
    End If
End Sub

Private Sub DrawPoint(ByVal x As Single, ByVal y As Single, _
    ByVal back_color As OLE_COLOR, ByVal fore_color As OLE_COLOR)
Const DIAMETER As Single = 30

    FillStyle = vbFSSolid
    FillColor = back_color
    Circle (x, y), DIAMETER, fore_color
    FillStyle = vbFSTransparent
End Sub

Private Function Min(ByVal x1 As Single, ByVal x2 As Single) As Single
    If x1 < x2 Then
        Min = x1
    Else
        Min = x2
    End If
End Function

' Find the points where the two circles intersect.
Private Function FindCircleCircleIntersections( _
    ByVal cx0 As Single, ByVal cy0 As Single, ByVal radius0 As Single, _
    ByVal cx1 As Single, ByVal cy1 As Single, ByVal radius1 As Single, _
    ByRef intersectionx1 As Single, ByRef intersectiony1 As Single, _
    ByRef intersectionx2 As Single, ByRef intersectiony2 As Single) As Integer
Dim dx, dy As Single
Dim dist, a, h, cx2, cy2 As Double

    ' Find the distance between the centers.
    dx = cx0 - cx1
    dy = cy0 - cy1
    dist = Sqr(dx * dx + dy * dy)

    ' See how many solutions there are.
    If (dist > radius0 + radius1) Then
        ' No solutions, the circles are too far apart.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
        Exit Function
    ElseIf (dist < Math.Abs(radius0 - radius1)) Then
        ' No solutions, one circle contains the other.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
    ElseIf ((dist = 0) And (radius0 = radius1)) Then
        ' No solutions, the circles coincide.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
    Else
        ' Find a and h.
        a = (radius0 * radius0 - _
            radius1 * radius1 + dist * dist) / (2 * dist)
        h = Sqr(radius0 * radius0 - a * a)

        ' Find P2.
        cx2 = cx0 + a * (cx1 - cx0) / dist
        cy2 = cy0 + a * (cy1 - cy0) / dist

        ' Get the points P3.
        intersectionx1 = CSng(cx2 + h * (cy1 - cy0) / dist)
        intersectiony1 = CSng(cy2 - h * (cx1 - cx0) / dist)
        intersectionx2 = CSng(cx2 - h * (cy1 - cy0) / dist)
        intersectiony2 = CSng(cy2 + h * (cx1 - cx0) / dist)

        ' See if we have 1 or 2 solutions.
        If (dist = radius0 + radius1) Then
            FindCircleCircleIntersections = 1
        Else
            FindCircleCircleIntersections = 2
        End If
        Exit Function
    End If
End Function
